package net.lemnik.eodsql;

import java.lang.annotation.Target;
import java.lang.annotation.Retention;
import java.lang.annotation.ElementType;
import java.lang.annotation.RetentionPolicy;

/**
 * <p>
 * The {@code AutoGeneratedKeys} annotation marks classes, or fields in the
 * data-object that are to be filled in from keys that are generated by an
 * {@link Update @Update} query. Fields and methods annotated by this class may
 * also be annotated by {@link ResultColumn @ResultColumn} for
 * {@link Select @Select} methods or to specify the column name for the field.
 * This annotation can be used in two ways, you can either annotate an entire
 * class as a "keys" class, as below:
 * </p><p><code>
 * <span style="color: #00aa00; font-weight: bold;">@AutoGeneratedKeys</span><br>
 * <span style="color: #0000ff;">public class</span> <b>PostKey</b> {<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #0000ff;">public</span> Long topicId;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #0000ff;">public</span> Long postId;<br>
 * }
 * </code></p><p>
 * Or you can annotate individual methods and fields in a class as "keys" columns.
 * As with the {@code @Select} annotation, only methods annotated with
 * {@link ResultColumn @ResultColumn} will be invoked. An example:
 * </p><p><code>
 * <span style="color: #0000ff;">public class</span> <b>Post</b> {<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #00aa00;
 *      font-weight: bold;">@AutoGeneratedKeys</span><br>
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #00aa00;
 *      font-weight: bold;">@ResultColumn</span>(<span style="color: #ffa536">"topic_id"</span>)<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #0000ff;">public</span> Long topicId;<br><br>
 *        
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #00aa00;
 *      font-weight: bold;">@AutoGeneratedKeys</span><br>
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #0000ff;">public</span> Long id;<br><br>
 *        
 * &nbsp;&nbsp;&nbsp;&nbsp;<span style="color: #0000ff;">public</span> String title;<br>
 * }
 * </code></p>
 *
 * @see Update @Update
 * @see ResultColumn @ResultColumn
 * 
 * @author Jason Morris
 */
@Target({ElementType.FIELD, ElementType.METHOD, ElementType.TYPE})
@Retention(RetentionPolicy.RUNTIME)
public @interface AutoGeneratedKeys {
}
