package net.lemnik.eodsql.impl;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.util.List;

import net.lemnik.eodsql.DataIterator;
import net.lemnik.eodsql.DataSet;
import net.lemnik.eodsql.spi.MethodImplementation;

/**
 * A factory for query method implementation parameters from dynamic invocation arguments. 
 *
 * @author Bernd Rinn
 */
interface DynamicParameterFactory {
    
    enum ReturnType { 
        DATASET(DataSet.class, false), 
        DATAITERATOR(DataIterator.class, false), 
        LIST(List.class, false),
        SINGLEROW(null, true);
        
        private final Class<?> rawType;
        
        private final boolean singleRow;
        
        ReturnType(Class<?> rawType, boolean singleRow) {
            this.rawType = rawType;
            this.singleRow = singleRow;
        }
        
        Class<?> getRawType() {
            return rawType;
        }
        
        boolean isSingleRow() {
            return singleRow;
        }
    }
    
    /**
     * Creates the annotation dynamically for the given method arguments. 
     */
    Annotation createAnnotation(final Object[] args);
    
    /**
     * Creates the base return type (e.g. the record type) for the given method arguments.
     */
    Class createBaseReturnType(final Object[] args);
    
    /**
     * Creates the implementation to invoke for the dynamic query.
     */
    MethodImplementation<? extends Annotation> createMethodImplementation(Method method, 
            Annotation annotation, Class[] queryParamTypes, Type returnType, Object[] args) 
                    throws ParseException;
    
    /**
     * Returns the return type of the query
     * <p>
     * Can be either a single row type, a collection, a data set or a data iterator.
     */
    ReturnType getReturnType();
    
    /**
     * Returns the SQL to execute for the given method arguments.
     */
    String getSql(Object[] args);

}