package net.lemnik.eodsql.impl;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Arrays;

/**
 * A key class of method implementations used for caching. 
 *
 * @author Bernd Rinn
 */
class MethodImplementationKey {
    final Method method;
    final Annotation annotation;
    final Class[] queryParamTypes;
    final Class baseReturnType;
    
    MethodImplementationKey(Method method,
        Annotation annotation, Class[] queryParamTypes, Class baseReturnType) {
        this.method = method;
        this.annotation = annotation;
        this.queryParamTypes = queryParamTypes;
        this.baseReturnType = baseReturnType;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
            + ((annotation == null) ? 0 : annotation.hashCode());
        result = prime * result
            + ((method == null) ? 0 : method.hashCode());
        result = prime * result + Arrays.hashCode(queryParamTypes);
        result = prime * result
            + ((baseReturnType == null) ? 0 : baseReturnType.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        MethodImplementationKey other = (MethodImplementationKey) obj;
        if (annotation == null) {
            if (other.annotation != null)
                return false;
        } else if (!annotation.equals(other.annotation))
            return false;
        if (method == null) {
            if (other.method != null)
                return false;
        } else if (!method.equals(other.method))
            return false;
        if (!Arrays.equals(queryParamTypes, other.queryParamTypes))
            return false;
        if (baseReturnType == null) {
            if (other.baseReturnType != null)
                return false;
        } else if (!baseReturnType.equals(other.baseReturnType))
            return false;
        return true;
    }
    
}